# Library imports
from typing import Dict, Any
import xml.etree.ElementTree as ET
from multiprocessing import Lock

# Project imports
from configg.backend import Backend


class XmlBackend(Backend):
    """ ini file backend for configg """

    def __init__(self, path: str):
        super().__init__(path)
        self.lock = Lock()

    def read(self) -> Dict[str, Any]:
        root = ET.parse(self.path).getroot()
        data = {}
        for child in root:
            data[child.tag] = {}
            for child2 in child:
                data[child.tag][child2.tag] = child2.text
        return data

    def write(self, data: Dict[str, Any]) -> None:
        sections = []
        for section, section_data in data.items():
            xml = "<{}>".format(section)
            for key, value in section_data.items():
                xml += "<{key}>{value}</{key}>".format(key=key, value=value)
            xml += "</{}>".format(section)
            sections.append(xml)
        xml_data = "<config>"
        for xml in sections:
            xml_data += xml
        xml_data += "</config>"
        self.lock.acquire(False)
        with open(self.path, "w+") as fp:
            fp.truncate()
            fp.seek(0)
            fp.write(xml_data)
        self.lock.release()





from configg.ini_backend import IniBackend
from configg.json_backend import JsonBackend
from configg.xml_backend import XmlBackend
from configg.configg import Configg
from configg.exceptions import *

INI_BACKEND = IniBackend
XML_BACKEND = XmlBackend
JSON_BACKEND = JsonBackend








# Library imports
from typing import Dict, Any
import configparser
from multiprocessing import Lock

# Project imports
from configg.backend import Backend


class IniBackend(Backend):
    """ ini file backend for configg """

    def __init__(self, path: str):
        super().__init__(path)
        self.lock = Lock()

    def read(self) -> Dict[str, Any]:
        parser = configparser.ConfigParser()
        parser.read(self.path)
        data = {}
        for section_name, section_dict in parser._sections.items():
            data[section_name] = section_dict
        return self._strip_quotes(data)

    def write(self, data: Dict[str, Any]) -> None:
        self.lock.acquire(False)
        with open(self.path, "w+") as fp:
            parser = configparser.ConfigParser()
            parser.read_dict(data)
            parser.write(fp)
        self.lock.release()



# Library imports
from __future__ import annotations
from typing import Dict, Any, TYPE_CHECKING

# Type check imports
if TYPE_CHECKING:
    from configg.configg import Configg

# Project imports
from configg.exceptions import *


class SectionView:

    def __init__(self, parent: Configg, data: dict):
        for key, value in data.items():
            if type(key) is not str:
                raise Exception
            if type(key) not in [str, int]:
                raise Exception
        self._parent = parent
        self._data = data

    def __getitem__(self, key) -> Any:
        """
        dict operator, provides dict-like access to data
        :param key: key
        :return: Value stored (or subsequent dict)
        """
        if key in self._data:
            return self._data[key]

    def __setitem__(self, key, value) -> None:
        """
        dict operator, provides dict-like access to write data
        :param key: key
        :param value:
        :return:
        """
        if self._parent.readonly:
            raise ReadOnlyError
        self._data[key] = value
        if self._parent.autocommit:
            self.commit()

    def __contains__(self, key) -> bool:
        """
        dict operator, returns true if key is present
        :param key: Key to check
        :return: key present?
        """
        return key in self._data

    def commit(self) -> None:
        """ Commits current configg data to file """
        self._parent.commit()

    def reload(self) -> None:
        """ Reloads configg data from file """
        self._parent.reload()

    def configg(self) -> Configg:
        """ :return: Return parent Configg instance """
        return self._parent

    def as_dict(self) -> dict:
        """ :return: Return data as pure python dictionary """
        return self._data




# Library imports
from typing import Dict, Any
import json
from multiprocessing import Lock

# Project imports
from configg.backend import Backend


class JsonBackend(Backend):
    """ ini file backend for configg """

    def __init__(self, path: str):
        super().__init__(path)
        self.lock = Lock()

    def read(self) -> Dict[str, Any]:
        with open(self.path, "r") as fp:
            return json.load(fp)

    def write(self, data: Dict[str, Any]) -> None:
        self.lock.acquire(False)
        with open(self.path, "w+") as fp:
            json.dump(data, fp)
        self.lock.release()



import setuptools

setuptools.setup(name='configg',
      version='v0.4-alpha',
      description='Config data manager',
      author='Oliver',
      author_email='mail.ok65@googlemail.com',
      url='https://github.com/ok65/Configg',
      packages=['configg'],
     )




from typing import Dict, Any


class Backend:

    def __init__(self, path: str):
        self.path = path

    def read(self) -> Dict[str, Any]:
        raise NotImplementedError

    def write(self, wr_dict: Dict[str, Any]) -> None:
        raise NotImplementedError

    @classmethod
    def _strip_quotes(cls, data: dict) -> dict:
        clean_dict = {}
        for key, value in data.items():
            if isinstance(value, dict):
                clean_dict[key] = cls._strip_quotes(value)
            else:
                clean_dict[key] = value.strip('"').strip("'")
        return clean_dict







# Library imports
from __future__ import annotations
from typing import TYPE_CHECKING, Optional, Dict, Any, List

# Type check imports
if TYPE_CHECKING:
    pass

# Project imports
from configg.exceptions import *
from configg.section_view import SectionView
from configg.ini_backend import IniBackend


class Configg:
    """
        Simple class that loads config data from a variety of formats (ini, json, xml etc), and presents them as a
        simple dictionary that can be read and written.

        Example:
            cfg = Configg("configg.ini")
            a = cfg.section_one["val_one"]
    """
    def __init__(self, path: str, data_backend=None, autocommit=False, readonly=False):
        """
        Configg constructor
        :param path: Path to config data file (it doesn't exist, will be created)
        :param data_backend: Backend data format (json, ini, xml etc) - defaults to ini
        :param autocommit: If true, writes to the dict will be written back to the config file
        :param readonly: Prevents modifying the dict if true
        """
        data_backend = data_backend or IniBackend
        self._backend = data_backend(path)
        self.readonly = readonly
        self.autocommit = autocommit
        self._sections = {}
        self.reload()

    @property
    def sections(self) -> List[str]:
        """ :return: List of section names """
        return list(self._sections.keys())

    def commit(self) -> None:
        """ Commits current configg data to file """
        self._backend.write(self._sections)

    def reload(self) -> None:
        """ Reloads configg data from file """
        self._sections = self._backend.read()

    def add_section(self, name: str, data: Optional[Dict[str, Any]] = None) -> SectionView:
        """
        Adds new section to configg data.
        :param name: Name of section (will overwrite existing section)
        :param data: Dict of data to store in section (if omitted, will create empty section)
        """
        self._sections[name] = data or {}
        return SectionView(self, self._sections[name])

    def remove_section(self, name: str) -> None:
        """
        Removes section from configg data
        :param name: Name of section
        """
        del self._sections[name]

    def __getattr__(self, item) -> SectionView:
        return SectionView(self, self._sections[item])
